package com.cosylab.vdct.graphics.objects;

/**
 * Copyright (c) 2002, Cosylab, Ltd., Control System Laboratory, www.cosylab.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer. 
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation 
 * and/or other materials provided with the distribution. 
 * Neither the name of the Cosylab, Ltd., Control System Laboratory nor the names
 * of its contributors may be used to endorse or promote products derived 
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

import java.awt.*;
import java.util.*;
import com.cosylab.vdct.Constants;
import com.cosylab.vdct.Settings;
import com.cosylab.vdct.graphics.*;

import com.cosylab.vdct.graphics.popup.*;
import com.cosylab.vdct.undo.CreateConnectorAction;
import com.cosylab.vdct.undo.DeleteConnectorAction;
import com.cosylab.vdct.undo.UndoManager;

import javax.swing.*;
import java.awt.event.*;

/**
 * Insert the type's description here.
 * Creation date: (29.1.2001 20:05:51)
 * @author Matej Sekoranja
 */
public class Connector extends VisibleObject implements Descriptable, InLink, Movable, OutLink, Popupable, Selectable {
	private static final String modeString = "Mode";
	private static final String normalString = "Normal";
	private static final String invisibleString = "Invisible";
	private static final String inputString = "External INPUT";
	private static final String outputString = "External OUTPUT";

	class PopupMenuHandler implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			String action = e.getActionCommand();
			if (action.equals(descriptionString)) {
				//setDescription();
				com.cosylab.vdct.events.CommandManager.getInstance().execute("RepaintWorkspace");
			}
			else if (action.equals(addConnectorString)) {
				addConnector();
				com.cosylab.vdct.events.CommandManager.getInstance().execute("RepaintWorkspace");
			}
			else if (action.equals(removeConnectorString)) {
				bypass();
				com.cosylab.vdct.events.CommandManager.getInstance().execute("RepaintWorkspace");
			}
			else if (action.equals(normalString)) {
				setMode(OutLink.NORMAL_MODE);
				com.cosylab.vdct.events.CommandManager.getInstance().execute("RepaintWorkspace");
			}
			else if (action.equals(invisibleString)) {
				setMode(OutLink.INVISIBLE_MODE);
				com.cosylab.vdct.events.CommandManager.getInstance().execute("RepaintWorkspace");
			}
			else if (action.equals(inputString)) {
				setMode(OutLink.EXTERNAL_INPUT_MODE);
				com.cosylab.vdct.events.CommandManager.getInstance().execute("RepaintWorkspace");
			}
			else if (action.equals(outputString)) {
				setMode(OutLink.EXTERNAL_OUTPUT_MODE);
				com.cosylab.vdct.events.CommandManager.getInstance().execute("RepaintWorkspace");
			}
			
		}
	}
	protected InLink inlink = null;
	protected OutLink outlink = null;
	protected String ID;
	protected boolean disconnected = false;
	protected int mode = OutLink.NORMAL_MODE;
	private static final String descriptionString = "Description";
	private static final String addConnectorString = "Add connector";
	private static final String removeConnectorString = "Remove connector";
/**
 * Insert the method's description here.
 * Creation date: (1.2.2001 17:22:29)
 * @param parent com.cosylab.vdct.graphics.objects.Record
 * @param outlink com.cosylab.vdct.graphics.objects.OutLink
 * @param inlink com.cosylab.vdct.graphics.objects.InLink
 */
public Connector(String id, LinkManagerObject parent, OutLink outlink, InLink inlink) {
	super(parent);
	setID(id);
	
	if (outlink instanceof VisibleObject)
		setColor(((VisibleObject)outlink).getColor());
	else
		setColor(Constants.FRAME_COLOR);

	//BugFix RT#12125&12122 by jbobnar
	boolean z = true;
	boolean outTurn = false;
	int outX = -1;
	int inX = -1;
	if (inlink != null) {
	    if (inlink instanceof Macro) {
	        z = inlink.isRight();
	    } else {
	        if (outlink != null) outX = outlink.getOutX();
	        inX = inlink.getInX();
	    }
	}
	if (outlink != null) {
	    outTurn = outlink.isRight();
	}
	//end bugfix
	
	if (inlink!=null) inlink.setOutput(this, outlink);
	if (outlink!=null) outlink.setInput(this);
	
	setInput(inlink); 
	setOutput(outlink, null);
	setWidth(Constants.CONNECTOR_WIDTH);
	setHeight(Constants.CONNECTOR_HEIGHT);

	/// !!! better initial layout
	// out of screen
	if ((inlink==null) && (outlink==null)) {
		setX(parent.getX()-(3*parent.getWidth())/2);
		setY(parent.getY()+parent.getHeight()/2);
	}
	else if (inlink==null) {
		setX(outlink.getOutX()-(3*getWidth())+2);
		setY(outlink.getOutY()+(3*getHeight())+2);
	}
	else if (outlink==null) {
		setX(inlink.getInX()-(3*getWidth())+2);
		setY(inlink.getInY()+(3*getHeight())+2);
	}
	else if (!inlink.getLayerID().equals(outlink.getLayerID()))
	{
		setX(parent.getX()-(3*parent.getWidth())/2);
		setY(parent.getY()+parent.getHeight()/2);
	}
	else {
	    // BugFix RT#12125&12122 by jbobnar
	    boolean setY = true;
	    int ir = inlink.getRightX();
	    int il = inlink.getLeftX();
	    int or = outlink.getRightX();
	    int ol = outlink.getLeftX();
	    int in = inlink.getInX();
	    int out = outlink.getOutX();

	    if (il > ol && ir > or && il <= or && !inlink.isRight()) {
	        if (outTurn != outlink.isRight())
	            setX((inX != in) ? inX : (inX < or) ? out : outX);
	        else         
	            setX((inX != in) || (outX != out) ? inX : out);
//	        System.out.println("1");
	    }
	    else if (ir < or && il > ol ) {
//	        System.out.println("2");
	        setX(out);
	    } 
	    else if ((ol > ir)){
//	        System.out.println("3");
	        if (outlink instanceof Port) {
	            setX(in + ir - il + 5);
	        } else 
	            setX(inX != in ? in + ir - il : in);
	    } else {
//	        System.out.println("4");
	        if (!z && !(outlink instanceof TemplateEPICSMacro)) {
	            setX(in + ir - il);
	        } else  {
	            if (outlink instanceof TemplateEPICSMacro && !(inlink instanceof Connector) && inlink.isRight()){
	                setX(out);
	            } else {
	                setY(inlink.getInY());
	                setY = false;
	                setX(in);
	            }
	        }

	    }
	    if (setY)
	        setY((inlink.getInY()+outlink.getOutY())/2);
	}

	// boundaries check
	ViewState view = ViewState.getInstance(getDsId());
	if (getX() < 0)
		setX(0);
	else if (getX() > view.getWidth())
		setX(view.getWidth());
	
	if (getY() < 0)
		setY(0);
	else if (getY() > view.getWidth())
		setY(view.getHeight());
	
	if (Settings.getInstance().getSnapToGrid())
		snapToGrid();
		
}
/**
 * Insert the method's description here.
 * Creation date: (29.1.2001 20:05:52)
 * @param visitor com.cosylab.vdct.graphics.objects.Visitor
 */
public void accept(Visitor visitor) {}
/**
 * Insert the method's description here.
 * Creation date: (4.2.2001 12:50:51)
 */
public Connector addConnector() {
    EPICSLink start = null;
    try { 
        start = (EPICSLink)EPICSLinkOut.getStartPoint(this);
    } catch (ClassCastException e) {
        //bugfix RT #12130 by jbobnar
        //adding connectors to port links
        start = (EPICSLink) EPICSLinkOut.getEndPoint(this);
    }

	if (start==null) return null;
	String id = EPICSLinkOut.generateConnectorID(start);
	String inlinkStr = "";
	String outlinkStr = getID();
	if (inlink!=null) inlinkStr = inlink.getID();
	Connector connector = new Connector(id, (LinkManagerObject)getParent(), this, getInput());
	getParent().addSubObject(id, connector);
	UndoManager.getInstance(getDsId()).addAction(new CreateConnectorAction(connector, inlinkStr, outlinkStr));
	return connector;
}
/**
 * Insert the method's description here.
 * Creation date: (4.2.2001 12:35:05)
 */
public void bypass() {
	if (!isDestroyed()) {
		super.destroy();

		String inlinkStr = "";
		String outlinkStr = "";
		if (inlink!=null) inlinkStr=inlink.getID();
		if (outlink!=null) outlinkStr=outlink.getID();

		InLink il = inlink;
		if (inlink!=null) inlink.setOutput(outlink, this);
		if (outlink!=null) outlink.setInput(il);

	
		getParent().removeObject(ID);
		setInput(null);
		setOutput(null, outlink);
		
		UndoManager.getInstance(getDsId()).addAction(new DeleteConnectorAction(this, inlinkStr, outlinkStr));
	}
}
/**
 * Insert the method's description here.
 * Creation date: (29.1.2001 20:05:51)
 * @return boolean
 * @param dx int
 * @param dy int
 */
public boolean checkMove(int dx, int dy) {
	ViewState view = ViewState.getInstance(getDsId());

	if ((getX()<-dx) || (getY()<-dy) || 
		(getX()>(view.getWidth()-getWidth()-dx)) || (getY()>(view.getHeight()-getHeight()-dy)) /*||
		// y must be between in/out !!!
		(((y+dy)-getOutY())*((y+dy)-getInY())>0)*/
		)
		return false;
	else
		return true;
}
/**
 * Insert the method's description here.
 * Creation date: (2.2.2001 23:00:51)
 * @return com.cosylab.vdct.graphics.objects.Connector.PopupMenuHandler
 */
private com.cosylab.vdct.graphics.objects.Connector.PopupMenuHandler createPopupmenuHandler() {
	return new PopupMenuHandler();
}
/**
 * Insert the method's description here.
 * Creation date: (1.2.2001 16:40:51)
 */
public void destroy() {
	if (!isDestroyed()) {
		super.destroy();
		getParent().removeObject(ID);

		String inlinkStr = "";
		String outlinkStr = "";
		if (inlink!=null) inlinkStr=inlink.getID();
		if (outlink!=null) outlinkStr=outlink.getID();
		UndoManager.getInstance(getDsId()).addAction(new DeleteConnectorAction(this, inlinkStr, outlinkStr));
		
		setInput(null);
		setOutput(null, outlink);
	}
}

/**
 * This method destroys connector (connector should not be used afer this method was called)
 * Creation date: (29.1.2001 20:05:51)
 */
public void disconnect(Linkable disconnector) {
	if (!disconnected && ((inlink==disconnector) || (outlink==disconnector))) {
		if (disconnector==inlink) inlink=null; //setInput(null);
		if (disconnector==outlink) outlink=null; //setOutput(null, outlink);
		if (((inlink==null) || inlink.isDisconnected()) &&
			((outlink==null) || outlink.isDisconnected()))
				destroy();
	}
}
/**
 * Insert the method's description here.
 * Creation date: (29.1.2001 20:05:52)
 * @param g java.awt.Graphics
 * @param hilited boolean
 */
protected void draw(java.awt.Graphics g, boolean hilited) {
	if (disconnected) return;
	
	ViewState view = ViewState.getInstance(getDsId());

	int rwidth = getRwidth();
	int rheight = getRheight();
	int rrx = getRx()-view.getRx()-rwidth/2;	// position is center
	int rry = getRy()-view.getRy()-rheight/2;
	
    boolean zoom = (view.getScale() < 1.0) && view.isZoomOnHilited() && view.isHilitedObject(this);
	
	if (!DsManager.getDrawingSurface(getDsId()).isPrinting()) {
		// clipping
		if ((!(rrx>view.getViewWidth()) || (rry>view.getViewHeight())
		    || ((rrx+rwidth)<0) || ((rry+rheight)<0))) {
	
			// fill or not to fill ?!!
	
			/*
			if (!hilited) g.setColor(getColor());
			else g.setColor((this==view.getHilitedObject()) ? 
							Constants.HILITE_COLOR : getColor());
			*/
//		    if (zoom) {
//		        rwidth /= Rscale;
//		        rheight /= Rscale;
//		        rrx -= (rwidth - getRwidth())/2;
//		        rry -= (rheight - getRheight())/2;
//		        if (view.getRx() < 0)
//		            rrx = rrx < 0 ? 2 : rrx;
//		        if (view.getRy() < 0) 
//		            rry = rry <= 0 ? 2 : rry;
//		        Rscale = 1.0;
//		    }
		    
			
			if (view.isSelected(this))
			{
				g.setColor(Constants.PICK_COLOR);
				g.fillRect(rrx, rry, rwidth, rheight);
			}
	
			Color c = getVisibleColor();
			if (hilited)
				c = (view.isHilitedObject(this)) ? Constants.HILITE_COLOR : c;
			g.setColor(c);

			if (inlink!=null || hilited) 
				g.drawRect(rrx, rry, rwidth, rheight);
			else if (getMode()!=EXTERNAL_OUTPUT_MODE && getMode()!=EXTERNAL_INPUT_MODE)
			{
				g.drawLine(rrx, rry, rrx+rwidth, rry+rheight);
				g.drawLine(rrx+rwidth, rry, rrx, rry+rheight);
			}
		
		}
	}

	if (/*!hilited &&*/ (inlink!=null || getMode()==EXTERNAL_OUTPUT_MODE || getMode()==EXTERNAL_INPUT_MODE)) {

		g.setColor(hilited && view.isHilitedObject(this) && !zoom ? Constants.HILITE_COLOR : getVisibleColor());

	
			if (inlink!=null){
				LinkDrawer.drawLink(g, view, this, inlink, getQueueCount(), 
									getOutX()<inlink.getInX());
			}
			else if (getOutput()!=null){
				LinkDrawer.drawLink(g, view, this, null, getQueueCount(), 
									getOutput().getOutX()<getX());
			}

	}  
	
}
/**
 * Insert the method's description here.
 * Creation date: (24.4.2001 18:04:05)
 * @return java.lang.String
 */
public java.lang.String getDescription() {
	return null;
}
/**
 * Insert the method's description here.
 * Creation date: (3.5.2001 16:43:08)
 * @return java.lang.String
 */
public java.lang.String getHashID() {
	return getID();
}
/**
 * Insert the method's description here.
 * Creation date: (1.2.2001 17:31:26)
 * @return java.lang.String
 */
public java.lang.String getID() {
	return ID;
}
/**
 * Insert the method's description here.
 * Creation date: (29.1.2001 20:05:52)
 * @return com.cosylab.vdct.graphics.objects.InLink
 */
public InLink getInput() {
	return inlink;
}
/**
 * Insert the method's description here.
 * Creation date: (29.1.2001 20:05:51)
 * @return int
 */
public int getInX() {
	return getX();
}
/**
 * Insert the method's description here.
 * Creation date: (29.1.2001 20:05:51)
 * @return int
 */
public int getInY() {
	return getY();
}
/**
 * Insert the method's description here.
 * Creation date: (3.2.2001 11:23:59)
 * @return java.util.Vector
 */
public java.util.Vector getItems() {
	Vector items = new Vector();

	ActionListener al = createPopupmenuHandler();

	JMenuItem descItem = new JMenuItem(descriptionString);
descItem.setEnabled(false); //!!!
	descItem.addActionListener(al);
	items.addElement(descItem);

	JMenuItem addItem = new JMenuItem(addConnectorString);
	addItem.addActionListener(al);
	items.addElement(addItem);
	
	JMenuItem removeItem = new JMenuItem(removeConnectorString);
	removeItem.addActionListener(al);
	items.addElement(removeItem);
	
	// modes
	items.addElement(new JSeparator());
	
	JMenu modeMenu = new JMenu(modeString);
	items.addElement(modeMenu);

	JRadioButtonMenuItem normalModeItem = new JRadioButtonMenuItem(normalString, getMode()==OutLink.NORMAL_MODE);
	normalModeItem.setEnabled(getMode()!=OutLink.NORMAL_MODE);
	normalModeItem.addActionListener(al);
	modeMenu.add(normalModeItem);

	JRadioButtonMenuItem invisibleModeItem = new JRadioButtonMenuItem(invisibleString, getMode()==OutLink.INVISIBLE_MODE);
	invisibleModeItem.setEnabled(getMode()!=OutLink.INVISIBLE_MODE && getInput()!=null && (getOutput()==null || getOutput().getMode()!=OutLink.INVISIBLE_MODE));
	invisibleModeItem.addActionListener(al);
	modeMenu.add(invisibleModeItem);
	
	modeMenu.add(new JSeparator());
	
	JRadioButtonMenuItem inputModeItem = new JRadioButtonMenuItem(inputString, getMode()==OutLink.EXTERNAL_INPUT_MODE);
	inputModeItem.setEnabled(getMode()!=OutLink.EXTERNAL_INPUT_MODE && getInput()==null);	// if endpoint
	inputModeItem.addActionListener(al);
	modeMenu.add(inputModeItem);

	JRadioButtonMenuItem outputModeItem = new JRadioButtonMenuItem(outputString, getMode()==OutLink.EXTERNAL_OUTPUT_MODE);
	outputModeItem.setEnabled(getMode()!=OutLink.EXTERNAL_OUTPUT_MODE && getInput()==null);	// if endpoint
	outputModeItem.addActionListener(al);
	modeMenu.add(outputModeItem);


	return items;
}
/**
 * Insert the method's description here.
 * Creation date: (29.1.2001 20:05:51)
 * @return java.lang.String
 */
public String getLayerID() {
	return getParent().getParent().toString();
}
/**
 * Insert the method's description here.
 * Creation date: (29.1.2001 20:05:51)
 * @return com.cosylab.vdct.graphics.objects.OutLink
 */
public OutLink getOutput() {
	return outlink;
}
/**
 * Insert the method's description here.
 * Creation date: (29.1.2001 20:05:52)
 * @return int
 */
public int getOutX() {
	return getX();
}
/**
 * Insert the method's description here.
 * Creation date: (29.1.2001 20:05:52)
 * @return int
 */
public int getOutY() {
	return getY();
}
/**
 * Insert the method's description here.
 * Creation date: (30.1.2001 14:50:26)
 * @return int
 */
public int getQueueCount() {
	if (!disconnected && (outlink!=null))
		return outlink.getQueueCount()+1;
	else
		return 0;
}
/**
 * Insert the method's description here.
 * Creation date: (29.1.2001 20:05:52)
 * @return boolean
 */
public boolean isConnectable() {
	return !disconnected;
}
/**
 * Insert the method's description here.
 * Creation date: (29.1.2001 20:05:51)
 * @return boolean
 */
public boolean isDisconnected() {
	return disconnected;
}
/**
 * Insert the method's description here.
 * Creation date: (29.1.2001 20:05:51)
 * @return boolean
 * @param dx int
 * @param dy int
 */
public boolean move(int dx, int dy) {
	if (checkMove(dx, dy)) {
		x+=dx; 
		/*
		if ((outlink!=null) && (inlink!=null))
			if (((y+dy)-getOutY())*((y+dy)-getInY())>0) {	// not in between
				// fix !!!?
			}
			else
				y+=dy;
		else*/
		y+=dy;
		
		revalidatePosition();
		if (inlink instanceof Connector) ((Connector)inlink).revalidatePosition();
		if (outlink instanceof Connector) ((Connector)outlink).revalidatePosition();
		return true;
	}
	else 
		return false;
}
/**
 * Insert the method's description here.
 * Creation date: (29.1.2001 20:05:52)
 */
public void revalidatePosition() {
  if (inlink!=null && outlink!=null && mode!=INVISIBLE_MODE && outlink.getMode()!=INVISIBLE_MODE) { //order is important
    if (getQueueCount()%2==0) 
        //setX(inlink.getInX());
		setX((inlink.getInX()+outlink.getOutX())/2);
    else
		setY((inlink.getInY()+outlink.getOutY())/2);
  }

  double  Rscale = getRscale();
  setRx((int)(getX()*Rscale));
  setRy((int)(getY()*Rscale));
  
}
/**
 * Insert the method's description here.
 * Creation date: (24.4.2001 18:04:05)
 * @param description java.lang.String
 */
public void setDescription(java.lang.String description) {}
/**
 * Insert the method's description here.
 * Creation date: (1.2.2001 17:31:26)
 * @param newID java.lang.String
 */
public void setID(java.lang.String newID) {
	ID = newID;
}
/**
 * Insert the method's description here.
 * Creation date: (29.1.2001 20:05:52)
 */
public void setInput(InLink input) {
	if (inlink==input) return;
	if (inlink!=null) inlink.disconnect(this);
	inlink=input;
	if ((inlink!=null) && (outlink!=null)) disconnected=false;
}
/**
 * Insert the method's description here.
 * Creation date: (29.1.2001 20:05:51)
 * @param id java.lang.String
 */
public void setLayerID(String id) {
	// not needed, dynamicaly retrieved via parent
}
/**
 * Insert the method's description here.
 * Creation date: (29.1.2001 20:05:51)
 * @param output com.cosylab.vdct.graphics.objects.OutLink
 * @param prevOutput com.cosylab.vdct.graphics.objects.OutLink
 */
public void setOutput(OutLink output, OutLink prevOutput) {
	if (outlink==output) return;
	if (outlink!=null) outlink.disconnect(this);
	outlink=output;
	if ((inlink!=null) || (outlink!=null)) disconnected=false;
	if (outlink instanceof VisibleObject)
		setColor(((VisibleObject)outlink).getColor());

}
/**
 * Insert the method's description here.
 * Creation date: (29.1.2001 20:05:52)
 */
protected void validate() {
  revalidatePosition();
  
  double Rscale = getRscale();
  setRwidth((int)(getWidth()*Rscale));
  setRheight((int)(getHeight()*Rscale));
}

/**
 * @see com.cosylab.vdct.graphics.objects.OutLink#getMode()
 */
public int getMode()
{
	return mode;
}

/**
 * Sets the mode.
 * @param mode The mode to set
 */
public void setMode(int mode)
{
	this.mode = mode;
}

/**
 * Default impmlementation for square (must be rescaled)
 * Creation date: (19.12.2000 20:20:20)
 * @return com.cosylab.visible.objects.VisibleObject
 * @param px int
 * @param py int
 */
public VisibleObject intersects(int px, int py) {
	int rwidth = getRwidth();
	int rheight = getRheight();
	int rx = getRx()-rwidth/2;	// position is center
	int ry = getRy()-rheight/2;
	if ((rx<=px) && (ry<=py) && 
			((rx+rwidth)>=px) && 
			((ry+rheight)>=py)) return this;
	else return null;
}

	/**
 * Default impmlementation for square (must be rescaled)
 * p1 is upper-left point
 * Creation date: (19.12.2000 20:20:20)
 * @return com.cosylab.visible.objects.VisibleObject
 * @param p1x int
 * @param p1y int
 * @param p2x int
 * @param p2y int
 */

public VisibleObject intersects(int p1x, int p1y, int p2x, int p2y) {
	int rwidth = getRwidth();
	int rheight = getRheight();
	int rx = getRx()-rwidth/2;	// position is center
	int ry = getRy()-rheight/2;
	if ((rx>=p1x) && (ry>=p1y) && 
			((rx+rwidth)<=p2x) && 
			((ry+rheight)<=p2y)) return this;
	else return null;
}

/**
 * @see com.cosylab.vdct.graphics.objects.OutLink#validateLink()
 */
public void validateLink()
{
}

public int getLeftX() {
	return getX();
}

public int getRightX() {
	return getX();
}

public boolean isRight() {
	return false;
}

//public String toString() {
//    return getID();
//}

}
